import types

import __builtin__
_default_stored = set(__builtin__.__dict__.keys())

_default_stored.update("""
""".split())

_default_stored.add(None) # special value for class definition

def _default_printer(line_no, argstr):
    print "line %d: %s" % (line_no, argstr)

def find_name_error(o, stored=_default_stored, printer=_default_printer):
    """Find possibility of NameError

    unsupported issue:
    "from foo import *"
    LOAD_ATTR(foo.bar)
    """
    from co2insts import co2insts
    from get_co import get_co

    co = get_co(o)
    stored = set(stored)
    stored.update(co.co_varnames)

    children = set()
    for ins in co2insts(co):
        if ins.opname.startswith("LOAD_"):
            if ins.opname == "LOAD_CONST":
                const = co.co_consts[ins.arg]
                if type(const) == types.CodeType:
                    children.add(const)
                continue
            if ins.opname == "LOAD_ATTR":
                continue
            
            if ins.argstr not in stored:
                #print o
                #from _dis import dis
                #dis(o)
                printer(ins.line_no, ins.argstr)

        if ins.opname.startswith("STORE_"):
            stored.add(ins.argstr)

    for c in children:
        find_name_error(c, stored, printer)

def recursive(dirname):
    import os
    printed_filenames = []
    def printer(line_no, argstr):
        if filename not in printed_filenames:
            print "***", path
            printed_filenames.append(filename)
        print "line %d: %s" % (line_no, argstr)

    
    for (dp, dn, fns) in os.walk(dirname):
        for filename in fns:
            if filename.endswith(".py"):
                path = os.path.join(dp, filename)
                find_name_error(
                    file(path),
                    printer=printer)


def watch(targets):
    import os, time
    def get_mtime(filename):
        return os.stat(filename).st_mtime

    old_mtime = {}
    while 1:
        time.sleep(0.3)
        for filename in targets:
            mtime = get_mtime(filename)
            if old_mtime.get(filename) < mtime:
                print "***", filename, "modified"
                old_mtime[filename] = mtime
                find_name_error(file(filename))

def main():
    import sys
    import optparse
    parser = optparse.OptionParser()
    parser.add_option("-r", "--recursive", dest="dirname",
                      help="find *.py under the directory", metavar="DIR")
    parser.add_option("-w", "--watch",
                      action="store_true", dest="watch", default=False,
                      help="watching target continuously(not implemented yet)")

    (options, args) = parser.parse_args()

    if options.watch:
        import os
        print "watch mode"
        if options.dirname:
            targets = []
            for (dp, dn, fns) in os.walk(options.dirname):
                for filename in fns:
                    if filename.endswith(".py"):
                        path = os.path.join(dp, filename)
                        targets.append(path)
        else:
            targets = sys.argv[1:]

        watch(targets)

    if options.dirname:
        recursive(options.dirname)
    else:
        for filename in sys.argv[1:]:
            find_name_error(file(filename))
    
if __name__ == "__main__":
    main()
