"""instruction.py
definition of Instruction object
"""

from opcode import (opname, HAVE_ARGUMENT, hasconst, 
    hasname, haslocal, hascompare, cmp_op)

def foo():
    print x
    
class Instruction():
    def __init__(self, value, cobj = None, **kw):
        self.cobj = cobj
        self.value = value
        self.opcode = value[0]
        self.opname = opname[self.opcode]
        self.length = len(value)
        if self.opcode >= HAVE_ARGUMENT:
            self.arg = value[1] + value[2] * 256
            self.high = value[1]
            self.low = value[2]
        self.updateArgstr()
        self.relyOn = []
        self.reachable = False
        self.jumpTo = None
        self.next = None
        self.prev = None
        self.pos = "UNDEF"
        self.is_line_start = False
        self.line_no = "UNDEF"

        for k in kw:
            self.__dict__[k] = kw[k]

    def setArg(self, v):
        self.arg = v
        self.high = v % 256
        self.low = v / 256
        self.value[1] = self.high
        self.value[2] = self.low
        self.updateArgstr()
        
    def updateArgstr(self):
        if self.opcode >= HAVE_ARGUMENT:
            if self.cobj == None:
                self.argstr = str(self.arg)
            elif self.opcode in hasconst:
                self.argstr = str(self.cobj.co_consts[self.arg])
            elif self.opcode in hasname:
                self.argstr = str(self.cobj.co_names[self.arg])
            elif self.opcode in haslocal:
                self.argstr = str(self.cobj.co_varnames[self.arg])
            elif self.opcode in hascompare:
                self.argstr = cmp_op[self.arg]
            elif self.opname == "CALL_FUNCTION":
                self.argstr = "%d, %d" % (self.high, self.low)
            elif self.opname in ["LOAD_DEREF", "STORE_DEREF",
                                 "LOAD_CLOSURE"]:
                if self.arg < len(self.cobj.co_cellvars):
                    self.argstr = str(self.cobj.co_cellvars[self.arg])
                else:
                    self.argstr = str(self.cobj.co_freevars[
                        self.arg - len(self.cobj.co_cellvars)])
            else:
                self.argstr = str(self.arg)
        else:
            self.argstr = None
        
    def get_arg(self):
        return self.arg

    def use(self, stack, num):
        assert isinstance(num, int)
        for i in range(num):
            self.relyOn.append(stack.pop())

    def push(self, stack, num):
        for i in range(num):
            stack.append(self)        

    def __repr__(self):
        if self.argstr != None:
            return "%s(%s)@%s" % (self.opname, self.argstr, self.pos)
        else:
            return "%s@%s" % (self.opname, self.pos)
    
    def __str__(self):
        result = repr(self)
        if self.relyOn:
            result += " rely on %r" % self.relyOn
        return result

